@@ -296,8 +296,10 @@ impl AmlContext {
 
             LevelType::Scope => Ok(true),
 
-            // TODO: can either of these contain devices?
+            // TODO: can any of these contain devices?
             LevelType::Processor => Ok(false),
+            LevelType::PowerResource => Ok(false),
+            LevelType::ThermalZone => Ok(false),
             LevelType::MethodLocals => Ok(false),
         })?;
 
@@ -713,10 +715,6 @@ pub enum AmlError {
     InvalidNameSeg,
     InvalidPkgLength,
     InvalidFieldFlags,
-    IncompatibleValueConversion {
-        current: AmlType,
-        target: AmlType,
-    },
     UnterminatedStringConstant,
     InvalidStringConstant,
     InvalidRegionSpace(u8),
@@ -783,6 +781,10 @@ pub enum AmlError {
     /*
      * Errors produced working with AML values.
      */
+    IncompatibleValueConversion {
+        current: AmlType,
+        target: AmlType,
+    },
     InvalidStatusObject,
     InvalidShiftLeft,
     InvalidShiftRight,
@@ -790,4 +792,18 @@ pub enum AmlError {
     FieldInvalidAddress,
     FieldInvalidAccessSize,
     TypeCannotBeCompared(AmlType),
+    /// Produced when the `Mid` operator is applied to a value of a type other than `Buffer` or `String`.
+    TypeCannotBeSliced(AmlType),
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_send_sync() {
+        // verify that AmlContext implements Send and Sync
+        fn test_send_sync<T: Send + Sync>() {}
+        test_send_sync::<AmlContext>();
+    }
 }
