@@ -97,7 +97,17 @@ where
     comment_scope(
         DebugVerbosity::AllScopes,
         "NamedObj",
-        choice!(def_op_region(), def_field(), def_method(), def_device(), def_processor(), def_mutex()),
+        choice!(
+            def_op_region(),
+            def_field(),
+            def_method(),
+            def_external(),
+            def_device(),
+            def_processor(),
+            def_power_res(),
+            def_thermal_zone(),
+            def_mutex()
+        ),
     )
 }
 
@@ -380,6 +390,20 @@ where
         .discard_result()
 }
 
+pub fn def_external<'a, 'c>() -> impl Parser<'a, 'c, ()>
+where
+    'c: 'a,
+{
+    /*
+     * DefExternal = 0x15 NameString ObjectType ArgumentCount
+     * ObjectType := ByteData
+     * ArgumentCount := ByteData (0 to 7)
+     */
+    opcode(opcode::DEF_EXTERNAL_OP)
+        .then(comment_scope(DebugVerbosity::Scopes, "DefExternal", name_string().then(take()).then(take())))
+        .discard_result()
+}
+
 pub fn def_device<'a, 'c>() -> impl Parser<'a, 'c, ()>
 where
     'c: 'a,
@@ -394,7 +418,11 @@ where
             pkg_length()
                 .then(name_string())
                 .map_with_context(|(length, name), context| {
-                    let resolved_name = try_with_context!(context, name.clone().resolve(&context.current_scope));
+                    let resolved_name = try_with_context!(context, name.resolve(&context.current_scope));
+                    try_with_context!(
+                        context,
+                        context.namespace.add_value(resolved_name.clone(), AmlValue::Device)
+                    );
                     try_with_context!(
                         context,
                         context.namespace.add_level(resolved_name.clone(), LevelType::Device)
@@ -466,6 +494,96 @@ where
         .discard_result()
 }
 
+pub fn def_power_res<'a, 'c>() -> impl Parser<'a, 'c, ()>
+where
+    'c: 'a,
+{
+    /*
+     * DefPowerRes := ExtOpPrefix 0x84 PkgLength NameString SystemLevel ResourceOrder TermList
+     * SystemLevel := ByteData
+     * ResourceOrder := WordData
+     */
+    ext_opcode(opcode::EXT_DEF_POWER_RES_OP)
+        .then(comment_scope(
+            DebugVerbosity::Scopes,
+            "DefPowerRes",
+            pkg_length()
+                .then(name_string())
+                .then(take())
+                .then(take_u16())
+                .map_with_context(|(((pkg_length, name), system_level), resource_order), context| {
+                    /*
+                     * `PowerResource` objects contain data within themselves, and can also have sub-objects,
+                     * so we add both a level for the sub-objects, and a value for the data.
+                     */
+                    let resolved_name = try_with_context!(context, name.resolve(&context.current_scope));
+                    try_with_context!(
+                        context,
+                        context.namespace.add_level(resolved_name.clone(), LevelType::PowerResource)
+                    );
+                    try_with_context!(
+                        context,
+                        context.namespace.add_value(
+                            resolved_name.clone(),
+                            AmlValue::PowerResource { system_level, resource_order }
+                        )
+                    );
+                    let previous_scope = context.current_scope.clone();
+                    context.current_scope = resolved_name;
+
+                    (Ok((previous_scope, pkg_length)), context)
+                })
+                .feed(move |(previous_scope, pkg_length)| {
+                    term_list(pkg_length).map(move |_| Ok(previous_scope.clone()))
+                })
+                .map_with_context(|previous_scope, context| {
+                    context.current_scope = previous_scope;
+                    (Ok(()), context)
+                }),
+        ))
+        .discard_result()
+}
+
+pub fn def_thermal_zone<'a, 'c>() -> impl Parser<'a, 'c, ()>
+where
+    'c: 'a,
+{
+    /*
+     * DefThermalZone := ExtOpPrefix 0x85 PkgLength NameString TermList
+     * TODO: we use this pattern a lot (move into scope, parse a term_list, move back out). Could we simplify into
+     * just a `feed` by passing a scope into term_list?
+     */
+    ext_opcode(opcode::EXT_DEF_THERMAL_ZONE_OP)
+        .then(comment_scope(
+            DebugVerbosity::Scopes,
+            "DefThermalZone",
+            pkg_length()
+                .then(name_string())
+                .map_with_context(|(pkg_length, name), context| {
+                    let resolved_name = try_with_context!(context, name.resolve(&context.current_scope));
+                    try_with_context!(
+                        context,
+                        context.namespace.add_value(resolved_name.clone(), AmlValue::ThermalZone)
+                    );
+                    try_with_context!(
+                        context,
+                        context.namespace.add_level(resolved_name.clone(), LevelType::ThermalZone)
+                    );
+
+                    let previous_scope = context.current_scope.clone();
+                    context.current_scope = resolved_name;
+
+                    (Ok((pkg_length, previous_scope)), context)
+                })
+                .feed(|(length, previous_scope)| term_list(length).map(move |_| Ok(previous_scope.clone())))
+                .map_with_context(|previous_scope, context| {
+                    context.current_scope = previous_scope;
+                    (Ok(()), context)
+                }),
+        ))
+        .discard_result()
+}
+
 pub fn def_mutex<'a, 'c>() -> impl Parser<'a, 'c, ()>
 where
     'c: 'a,
