@@ -18,8 +18,12 @@ use crate::{
     AmlError,
     DebugVerbosity,
 };
-use alloc::{vec, vec::Vec};
-use core::{cmp::Ordering, convert::TryInto};
+use alloc::{
+    string::{String, ToString},
+    vec,
+    vec::Vec,
+};
+use core::{cmp::Ordering, convert::TryInto, mem};
 
 /// Type 2 opcodes return a value and so can be used in expressions.
 pub fn type2_opcode<'a, 'c>() -> impl Parser<'a, 'c, AmlValue>
@@ -44,6 +48,8 @@ where
             def_add(),
             def_and(),
             def_buffer(),
+            def_concat(),
+            def_concat_res(),
             def_l_equal(),
             def_l_greater(),
             def_l_greater_equal(),
@@ -51,6 +57,7 @@ where
             def_l_less_equal(),
             def_l_not_equal(),
             def_l_or(),
+            def_mid(),
             def_package(),
             def_shift_left(),
             def_shift_right(),
@@ -147,6 +154,108 @@ where
         .map(|((), buffer)| Ok(AmlValue::Buffer(buffer)))
 }
 
+pub fn def_concat<'a, 'c>() -> impl Parser<'a, 'c, AmlValue>
+where
+    'c: 'a,
+{
+    /*
+     * DefConcat := 0x73 Data Data Target
+     * Data := TermArg => ComputationalData
+     */
+    opcode(opcode::DEF_CONCAT_OP)
+        .then(comment_scope(
+            DebugVerbosity::AllScopes,
+            "DefConcat",
+            term_arg().then(term_arg()).then(target()).map_with_context(|((left, right), target), context| {
+                let result = match left.as_concat_type() {
+                    AmlValue::Integer(left) => {
+                        let right = try_with_context!(context, right.as_integer(context));
+
+                        let mut buffer = Vec::with_capacity(mem::size_of::<u64>() * 2);
+                        buffer.extend_from_slice(&left.to_le_bytes());
+                        buffer.extend_from_slice(&right.to_le_bytes());
+
+                        AmlValue::Buffer(buffer)
+                    }
+                    AmlValue::Buffer(mut left) => {
+                        left.extend(try_with_context!(context, right.as_buffer(context)));
+                        AmlValue::Buffer(left)
+                    }
+                    AmlValue::String(left) => {
+                        let right = match right.as_concat_type() {
+                            AmlValue::String(right) => right,
+                            AmlValue::Integer(_) => try_with_context!(context, right.as_string(context)),
+                            AmlValue::Buffer(_) => try_with_context!(context, right.as_string(context)),
+                            _ => panic!("Invalid type returned from `as_concat_type`"),
+                        };
+                        AmlValue::String(left + &right)
+                    }
+                    _ => panic!("Invalid type returned from `as_concat_type`"),
+                };
+
+                try_with_context!(context, context.store(target, result.clone()));
+                (Ok(result), context)
+            }),
+        ))
+        .map(|((), result)| Ok(result))
+}
+
+pub fn def_concat_res<'a, 'c>() -> impl Parser<'a, 'c, AmlValue>
+where
+    'c: 'a,
+{
+    /*
+     * DefConcatRes := 0x84 BufData BufData Target
+     * BufData := TermArg => Buffer
+     */
+    opcode(opcode::DEF_CONCAT_RES_OP)
+        .then(comment_scope(
+            DebugVerbosity::AllScopes,
+            "DefConcatRes",
+            term_arg().then(term_arg()).then(target()).map_with_context(|((left, right), target), context| {
+                let left = try_with_context!(context, left.as_buffer(context));
+                let right = try_with_context!(context, right.as_buffer(context));
+
+                if left.len() == 1 || right.len() == 1 {
+                    return (Err(Propagate::Err(AmlError::ResourceDescriptorTooShort)), context);
+                }
+
+                /*
+                 * `left` and `right` are buffers of resource descriptors, which we're trying to concatenate into a
+                 * new, single buffer containing all of the descriptors from the source buffers. We need to strip
+                 * off the end tags (2 bytes from each buffer), and then add our own end tag.
+                 *
+                 * XXX: either buffer may be empty (contains no tags), and so our arithmetic has to be careful.
+                 */
+                let result = {
+                    let mut result =
+                        Vec::with_capacity(left.len().saturating_sub(2) + right.len().saturating_sub(2) + 2);
+                    result.extend_from_slice(if left.len() == 0 { &[] } else { &left[..(left.len() - 2)] });
+                    result.extend_from_slice(if right.len() == 0 { &[] } else { &right[..(right.len() - 2)] });
+
+                    /*
+                     * Construct a new end tag, including a new checksum:
+                     *    | Bits        | Field             | Value                     |
+                     *    |-------------|-------------------|---------------------------|
+                     *    | 0-2         | Length - n bytes  | 1 (for checksum)          |
+                     *    | 3-6         | Small item type   | 0x0f = end tag descriptor |
+                     *    | 7           | 0 = small item    | 0                         |
+                     */
+                    result.push(0b01111001);
+                    result.push(
+                        result.iter().fold(0u8, |checksum, byte| checksum.wrapping_add(*byte)).wrapping_neg(),
+                    );
+
+                    AmlValue::Buffer(result)
+                };
+
+                try_with_context!(context, context.store(target, result.clone()));
+                (Ok(result), context)
+            }),
+        ))
+        .map(|((), result)| Ok(result))
+}
+
 fn def_l_or<'a, 'c>() -> impl Parser<'a, 'c, AmlValue>
 where
     'c: 'a,
@@ -286,6 +395,60 @@ where
         .map(|(((), ()), result)| Ok(result))
 }
 
+fn def_mid<'a, 'c>() -> impl Parser<'a, 'c, AmlValue>
+where
+    'c: 'a,
+{
+    /*
+     * DefMid := 0x9e MidObj TermArg TermArg Target
+     * MidObj := TermArg => Buffer | String
+     */
+    opcode(opcode::DEF_MID_OP)
+        .then(comment_scope(
+            DebugVerbosity::AllScopes,
+            "DefMid",
+            term_arg().then(term_arg()).then(term_arg()).then(target()).map_with_context(
+                |(((source, index), length), target), context| {
+                    let index = try_with_context!(context, index.as_integer(context)) as usize;
+                    let length = try_with_context!(context, length.as_integer(context)) as usize;
+
+                    let result = try_with_context!(
+                        context,
+                        match source {
+                            AmlValue::Buffer(bytes) => {
+                                if index >= bytes.len() {
+                                    Ok(AmlValue::Buffer(vec![]))
+                                } else if (index + length) >= bytes.len() {
+                                    Ok(AmlValue::Buffer(bytes[index..].to_vec()))
+                                } else {
+                                    Ok(AmlValue::Buffer(bytes[index..(index + length)].to_vec()))
+                                }
+                            }
+                            /*
+                             * XXX: The spec conflates characters and bytes, so we effectively ignore unicode and do
+                             * this bytewise, to hopefully match other implementations.
+                             */
+                            AmlValue::String(string) => {
+                                if index >= string.len() {
+                                    Ok(AmlValue::String(String::new()))
+                                } else if (index + length) >= string.len() {
+                                    Ok(AmlValue::String(string[index..].to_string()))
+                                } else {
+                                    Ok(AmlValue::String(string[index..(index + length)].to_string()))
+                                }
+                            }
+                            _ => Err(AmlError::TypeCannotBeSliced(source.type_of())),
+                        }
+                    );
+
+                    try_with_context!(context, context.store(target, result.clone()));
+                    (Ok(result), context)
+                },
+            ),
+        ))
+        .map(|((), result)| Ok(result))
+}
+
 pub fn def_package<'a, 'c>() -> impl Parser<'a, 'c, AmlValue>
 where
     'c: 'a,
