@@ -1,5 +1,9 @@
 use crate::{misc::ArgNum, AmlContext, AmlError, AmlHandle, AmlName};
-use alloc::{string::String, sync::Arc, vec::Vec};
+use alloc::{
+    string::{String, ToString},
+    sync::Arc,
+    vec::Vec,
+};
 use bit_field::BitField;
 use core::{cmp, fmt, fmt::Debug};
 
@@ -138,6 +142,7 @@ pub enum AmlType {
     DebugObject,
     Event,
     FieldUnit,
+    Device,
     Integer,
     Method,
     Mutex,
@@ -187,6 +192,7 @@ pub enum AmlValue {
         offset: u64,
         length: u64,
     },
+    Device,
     Method {
         flags: MethodFlags,
         code: MethodCode,
@@ -201,6 +207,11 @@ pub enum AmlValue {
         sync_level: u8,
     },
     Package(Vec<AmlValue>),
+    PowerResource {
+        system_level: u8,
+        resource_order: u16,
+    },
+    ThermalZone,
 }
 
 impl AmlValue {
@@ -231,11 +242,14 @@ impl AmlValue {
             AmlValue::String(_) => AmlType::String,
             AmlValue::OpRegion { .. } => AmlType::OpRegion,
             AmlValue::Field { .. } => AmlType::FieldUnit,
+            AmlValue::Device => AmlType::Device,
             AmlValue::Method { .. } => AmlType::Method,
             AmlValue::Buffer(_) => AmlType::Buffer,
             AmlValue::Processor { .. } => AmlType::Processor,
             AmlValue::Mutex { .. } => AmlType::Mutex,
             AmlValue::Package(_) => AmlType::Package,
+            AmlValue::PowerResource { .. } => AmlType::PowerResource,
+            AmlValue::ThermalZone => AmlType::ThermalZone,
         }
     }
 
@@ -250,6 +264,7 @@ impl AmlValue {
     pub fn as_integer(&self, context: &AmlContext) -> Result<u64, AmlError> {
         match self {
             AmlValue::Integer(value) => Ok(*value),
+            AmlValue::Boolean(value) => Ok(if *value { u64::max_value() } else { 0 }),
 
             AmlValue::Buffer(ref bytes) => {
                 /*
@@ -296,6 +311,34 @@ impl AmlValue {
         }
     }
 
+    /// Converts an `AmlValue` to the representation that should be used when concatenating it with other values,
+    /// primarily by the `DefConcat` opcode. This will always produce a `AmlValue::Integer`, `AmlValue::String`, or
+    /// `AmlValue::Buffer`, with other types being converted to strings containing the name of their type.
+    pub fn as_concat_type(&self) -> AmlValue {
+        match self.type_of() {
+            AmlType::Integer => self.clone(),
+            AmlType::String => self.clone(),
+            AmlType::Buffer => self.clone(),
+
+            AmlType::Uninitialized => AmlValue::String("[Uninitialized]".to_string()),
+            AmlType::BufferField => AmlValue::String("[Buffer Field]".to_string()),
+            AmlType::DdbHandle => AmlValue::String("[Ddb Handle]".to_string()),
+            AmlType::DebugObject => AmlValue::String("[Debug Object]".to_string()),
+            AmlType::Event => AmlValue::String("[Event]".to_string()),
+            AmlType::FieldUnit => AmlValue::String("[Field]".to_string()),
+            AmlType::Device => AmlValue::String("[Device]".to_string()),
+            AmlType::Method => AmlValue::String("[Control Method]".to_string()),
+            AmlType::Mutex => AmlValue::String("[Mutex]".to_string()),
+            AmlType::ObjReference => AmlValue::String("[Obj Reference]".to_string()),
+            AmlType::OpRegion => AmlValue::String("[Operation Region]".to_string()),
+            AmlType::Package => AmlValue::String("[Package]".to_string()),
+            AmlType::Processor => AmlValue::String("[Processor]".to_string()),
+            AmlType::PowerResource => AmlValue::String("[Power Resource]".to_string()),
+            AmlType::RawDataBuffer => AmlValue::String("[Raw Data Buffer]".to_string()),
+            AmlType::ThermalZone => AmlValue::String("[Thermal Zone]".to_string()),
+        }
+    }
+
     /// Turns an `AmlValue` returned from a `_STA` method into a `StatusObject`. Should only be called for values
     /// returned from `_STA`. If you need a `StatusObject`, but the device does not have a `_STA` method, use
     /// `StatusObject::default()`.
@@ -526,11 +569,6 @@ mod tests {
         assert_eq!(AmlValue::Integer(11).cmp(AmlValue::Integer(11), &mut context), Ok(Ordering::Equal));
         assert_eq!(AmlValue::Integer(8362836690).cmp(AmlValue::Integer(1), &mut context), Ok(Ordering::Greater));
 
-        assert_eq!(
-            AmlValue::Integer(4).cmp(AmlValue::Boolean(true), &mut context),
-            Err(AmlError::IncompatibleValueConversion { current: AmlType::Integer, target: AmlType::Integer })
-        );
-
         // TODO: test the other combinations too, as well as conversions to the correct types for the second operand
     }
 }
